/* BDF - Copyright (c) 2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi.commands;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.multi.Multi;
import fr.exemole.bdfserver.multi.MultiUtils;
import java.text.ParseException;
import net.fichotheque.SubsetKey;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FichothequeDuplicationCommand {

    public final static String COMMAND_NAME = "FichothequeDuplication";
    public final static String NEWFICHOTHEQUE_PARAMNAME = "newfichotheque";
    public final static String SOURCE_PARAMNAME = "source";
    public final static String INCLUDE_SUBSET_PARAMNAME = "include_subset";
    public final static String EXCLUDE_SUBSET_PARAMNAME = "exclude_subset";
    public final static String INCLUDE_ITEMS_PARAMNAME = "include_items";
    public final static String EXCLUDE_ITEMS_PARAMNAME = "exclude_items";
    private final Multi multi;
    private final RequestMap requestMap;

    public FichothequeDuplicationCommand(Multi multi, RequestMap requestMap) {
        this.multi = multi;
        this.requestMap = requestMap;
    }

    public CommandMessage doCommand() {
        try {
            String name = run();
            return LogUtils.done("_ done.multi.fichothequecreation", name);
        } catch (ErrorMessageException eme) {
            return eme.getErrorMessage();
        }
    }

    private String run() throws ErrorMessageException {
        String name = MultiUtils.getFichothequeName(requestMap, NEWFICHOTHEQUE_PARAMNAME);
        if (multi.getExistingNameSet().contains(name)) {
            throw new ErrorMessageException("_ error.existing.fichotheque", name);
        }
        multi.createFichotheque(getCreationParameters(name));
        return name;
    }

    private CreationParameters getCreationParameters(String name) throws ErrorMessageException {
        BdfServer sourceBdfServer = getSourceBdfServer();
        CreationSubsetEligibility creationSubsetEligibility = new CreationSubsetEligibility();
        checkEligility(creationSubsetEligibility, CreationSubsetEligibility.ITEMS, INCLUDE_ITEMS_PARAMNAME, true);
        checkEligility(creationSubsetEligibility, CreationSubsetEligibility.ITEMS, EXCLUDE_ITEMS_PARAMNAME, false);
        checkEligility(creationSubsetEligibility, CreationSubsetEligibility.SUBSET, INCLUDE_SUBSET_PARAMNAME, true);
        checkEligility(creationSubsetEligibility, CreationSubsetEligibility.SUBSET, EXCLUDE_SUBSET_PARAMNAME, false);
        return CreationParameters.initDuplicate(name, sourceBdfServer, creationSubsetEligibility);
    }

    private BdfServer getSourceBdfServer() throws ErrorMessageException {
        String source = requestMap.getParameter(SOURCE_PARAMNAME);
        if (source == null) {
            throw new ErrorMessageException("_ error.empty.mandatoryparameter", SOURCE_PARAMNAME);
        }
        BdfServer sourceBdfServer = multi.getBdfServer(source);
        if (sourceBdfServer == null) {
            throw new ErrorMessageException("_ error.unknown.fichotheque", source);
        }
        return sourceBdfServer;
    }

    private void checkEligility(CreationSubsetEligibility creationSubsetEligibility, short type, String paramName, boolean include) {
        String[] values = requestMap.getParameterValues(paramName);
        if (values == null) {
            return;
        }
        for (String value : values) {
            String[] tokens = StringUtils.getTechnicalTokens(value, true);
            for (String token : tokens) {
                try {
                    SubsetKey subsetKey = SubsetKey.parse(token);
                    creationSubsetEligibility.add(subsetKey, type, include);
                } catch (ParseException pe) {
                    try {
                        short category = SubsetKey.categoryToShort(token);
                        creationSubsetEligibility.add(category, type, include);
                    } catch (IllegalArgumentException iae) {

                    }
                }
            }
        }
    }


}
