/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage.engine;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.balayage.BalayageMode;
import fr.exemole.bdfserver.api.balayage.FileGenerationListener;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import java.io.File;
import java.util.List;
import java.util.function.Predicate;
import javax.xml.transform.URIResolver;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.tools.corpus.FichesBuilder;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.fichotheque.corpus.SortConstants;
import net.fichotheque.corpus.Fiches;


/**
 *
 * @author Vincent Calame
 */
public class BalayageParameters {

    private final Balayage balayage;
    private final BdfServer bdfServer;
    private final PathConfiguration pathConfiguration;
    private BalayageMode balayageMode;
    private BalayageLog balayageLog;
    private File balayageRootDirectory;
    private FileGenerationListener fileGenerationListener;
    private boolean extractionContextInit = false;
    private ExtractionContext extractionContext;
    private URIResolver customURIResolver;

    public BalayageParameters(Balayage balayage, BdfServer bdfServer, PathConfiguration pathConfiguration) {
        this.balayage = balayage;
        this.bdfServer = bdfServer;
        this.pathConfiguration = pathConfiguration;
    }

    public BalayageMode getBalayageMode() {
        return balayageMode;
    }

    public void setBalayageMode(BalayageMode balayageMode) {
        this.balayageMode = balayageMode;
    }

    public BalayageLog getBalayageLog() {
        return balayageLog;
    }

    public void setBalayageLog(BalayageLog balayageLog) {
        this.balayageLog = balayageLog;
    }

    public Balayage getBalayage() {
        return balayage;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public PathConfiguration getPathConfiguration() {
        return pathConfiguration;
    }

    public File getBalayageRootDirectory() {
        return balayageRootDirectory;
    }

    public void setBalayageRootDirectory(File balayageRootDirectory) {
        this.balayageRootDirectory = balayageRootDirectory;
    }

    public ExtractionContext getExtractionContext() {
        if (extractionContextInit) {
            return extractionContext;
        }
        LangContext langContext = balayage.getLangContext();
        if (langContext == null) {
            langContext = ConfigurationUtils.toLangContext(bdfServer.getLangConfiguration(), false);
        }
        Fiches fiches = null;
        Predicate<FicheMeta> fichePredicate = null;
        List<FicheQuery> ficheQueryList = balayage.getGlobalFicheQueryList();
        if (!ficheQueryList.isEmpty()) {
            Lang defaultLang = LocalisationUtils.getDefaultLang(langContext);
            SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, defaultLang)
                    .setSubsetAccessPredicate(EligibilityUtils.ALL_SUBSET_PREDICATE)
                    .toSelectionContext();
            FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext).addAll(ficheQueryList).toFicheSelector();
            fiches = FichesBuilder.build(SortConstants.NONE)
                    .populate(ficheSelector)
                    .toFiches();
            fichePredicate = fiches;
        }
        extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, langContext)
                .setSubsetAccessPredicate(EligibilityUtils.ALL_SUBSET_PREDICATE)
                .setFichePredicate(fichePredicate)
                .setSelectedFiches(fiches)
                .toExtractionContext();
        extractionContextInit = true;
        return extractionContext;
    }

    /**
     * @return the fileGenerationListener
     */
    public FileGenerationListener getFileGenerationListener() {
        return fileGenerationListener;
    }

    /**
     * @param fileGenerationListener the fileGenerationListener to set
     */
    public void setFileGenerationListener(FileGenerationListener fileGenerationListener) {
        this.fileGenerationListener = fileGenerationListener;
    }

    public URIResolver getCustomURIResolver() {
        return customURIResolver;
    }

    public void setCustomURIResolver(URIResolver customURIResolver) {
        this.customURIResolver = customURIResolver;
    }

}
