/* BdfServer - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation.compilers;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.exportation.transformation.StreamTemplateFactory;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.ParseException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import net.fichotheque.exportation.table.Cell;
import net.fichotheque.exportation.transformation.StreamTemplate;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.namespaces.TransformationSpace;
import net.fichotheque.tools.exportation.transformation.TemplateContentDescriptionBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;


/**
 *
 * @author Vincent Calame
 */
public class PropertiesOdStreamTemplateCompiler extends StreamTemplateCompiler {

    private final static String TEMPLATE_ODT = "template.odt";
    private final static String CONTENT_XML = "content.xml";

    public PropertiesOdStreamTemplateCompiler(BdfServer bdfServer, PathConfiguration pathConfiguration, TemplateKey templateKey, MessageHandler messageHandler) {
        super(bdfServer, pathConfiguration, templateKey, messageHandler);
    }

    @Override
    public StreamTemplateFactory compile(TemplateStorage.Unit storageUnit) throws CompilationException {
        Contents contents = new Contents(storageUnit);
        contents.readZip();
        List<Object> partList;
        try {
            partList = PropertiesOdParser.parse(contents.contentString);
        } catch (ParseException pe) {
            throw new CompilationException();
        }
        PropertiesStreamTemplate streamTemplate = new PropertiesStreamTemplate(contents.map, partList, getTableExportName(storageUnit));
        return new PropertiesStreamTemplateFactory(streamTemplate);
    }


    private class Contents {

        private final TemplateStorage.Unit storageUnit;
        private final Map<String, byte[]> map = new LinkedHashMap<String, byte[]>();
        private String contentString = "";

        private Contents(TemplateStorage.Unit storageUnit) {
            this.storageUnit = storageUnit;
        }

        private void readZip() throws CompilationException {
            try (ZipInputStream zis = new ZipInputStream(getZipInputStream(TEMPLATE_ODT))) {
                TemplateContentDescriptionBuilder templateContentDescriptionBuilder = newTemplateContentDescriptionBuilder(TEMPLATE_ODT, true, true);
                ZipEntry zipEntry;
                while ((zipEntry = zis.getNextEntry()) != null) {
                    String name = zipEntry.getName();
                    if (name.equals(CONTENT_XML)) {
                        contentString = IOUtils.toString(zis, "UTF-8");
                    } else {
                        byte[] bytes = IOUtils.toByteArray(zis);
                        map.put(name, bytes);
                    }
                }
            } catch (IOException ioe) {
                addError("_ error.exception.transformation_io", TEMPLATE_ODT, ioe.getMessage());
                throw new CompilationException();
            }
        }

        private InputStream getZipInputStream(String zipName) throws CompilationException {
            InputStream inputStream;
            try {
                inputStream = TemplateCompilerUtils.getContentInputStream(storageUnit, zipName);
            } catch (IOException ioe) {
                addError("_ error.exception.transformation_io", zipName, ioe.getMessage());
                throw new CompilationException();
            }
            if (inputStream == null) {
                addError("_ error.empty.transformation.ressource", zipName);
                throw new CompilationException();
            }
            return inputStream;
        }

    }


    private class PropertiesStreamTemplateFactory implements StreamTemplateFactory {

        private final StreamTemplate streamTemplate;

        private PropertiesStreamTemplateFactory(StreamTemplate streamTemplate) {
            this.streamTemplate = streamTemplate;
        }

        @Override
        public StreamTemplate newInstance() {
            return streamTemplate;
        }

    }


    private class PropertiesStreamTemplate implements StreamTemplate.Properties {

        private final Map<String, byte[]> map;
        private final List<Object> partList;
        private final String tableExportName;

        private PropertiesStreamTemplate(Map<String, byte[]> map, List<Object> partList, String tableExportName) {
            this.map = map;
            this.partList = partList;
            this.tableExportName = tableExportName;
        }

        @Override
        public String getMimeType() {
            return MimeTypeUtils.ODT;
        }

        @Override
        public String getTableExportName() {
            return tableExportName;
        }

        @Override
        public void fill(Cell[] cellArray, OutputStream outputStream) throws IOException {
            try (ZipOutputStream zos = new ZipOutputStream(outputStream)) {
                for (Map.Entry<String, byte[]> entry : map.entrySet()) {
                    ZipEntry zipEntry = new ZipEntry(entry.getKey());
                    zos.putNextEntry(zipEntry);
                    try (InputStream is = new ByteArrayInputStream(entry.getValue())) {
                        IOUtils.copy(is, zos);
                    }
                    zos.closeEntry();
                }
                ZipEntry zipEntry = new ZipEntry(CONTENT_XML);
                zos.putNextEntry(zipEntry);
                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(zos, "UTF-8"));
                writeContent(cellArray, writer);
                writer.flush();
                zos.closeEntry();
            }
        }

        private void writeContent(Cell[] cellArray, Writer writer) throws IOException {
            for (Object part : partList) {
                if (part instanceof String) {
                    writer.write((String) part);
                } else {
                    PropertyPart propertyPart = (PropertyPart) part;
                    String name = propertyPart.getName();
                    boolean done = false;
                    for (Cell cell : cellArray) {
                        if (cell.getColDef().getColName().equals(name)) {
                            Object value = cell.getValue();
                            if (value != null) {
                                if (value instanceof String) {
                                    String[] tokens = StringUtils.getLineTokens((String) value, StringUtils.NOTCLEAN);
                                    boolean next = false;
                                    for (String token : tokens) {
                                        if (next) {
                                            writer.write("<text:line-break/>");
                                        } else {
                                            next = true;
                                        }
                                        XMLUtils.writeEscape(writer, token);
                                    }
                                } else {
                                    XMLUtils.writeEscape(writer, value.toString());
                                }
                            }
                            done = true;
                            break;
                        }
                    }
                    if (!done) {
                        writer.write("????");
                        XMLUtils.writeEscape(writer, name);
                        writer.write("????");
                    }
                }
            }
        }

    }

    private static String getTableExportName(TemplateStorage.Unit storageUnit) {
        Attribute attribute = storageUnit.getTemplateDef().getAttributes().getAttribute(TransformationSpace.TABLEEXPORT_KEY);
        if (attribute != null) {
            return attribute.getFirstValue();
        } else {
            return "";
        }
    }

}
