/* BdfServer - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.storage.StorageContent;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.exportation.transformation.TemplateDef;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.tools.exportation.transformation.TemplateDefBuilder;
import net.fichotheque.xml.ExtractionDefXMLPart;
import net.mapeadores.opendocument.io.OdXML;


/**
 *
 * @author Vincent Calame
 */
class XsltOdtUnit {

    private XsltOdtUnit() {
    }

    static TemplateStorage.Unit build(BdfServer bdfServer, TemplateKey templateKey, boolean withExtractionDef) {
        if (templateKey.isSimpleTemplate()) {
            throw new IllegalArgumentException("templateKey is simple template key");
        } else if (!templateKey.getExtension().equals("odt")) {
            throw new IllegalArgumentException("templateKey.getExtension != odt");
        }
        String xsltString = getXslt(bdfServer, templateKey.getTransformationKey());
        if (xsltString == null) {
            return null;
        }
        TemplateDef templateDef = getTemplateDef(templateKey);
        List<StorageContent> list = new ArrayList<StorageContent>();
        list.add(StorageUtils.toStorageContent("content.xsl", xsltString));
        list.add(StorageUtils.toStorageContent("styles.css", DefaultTemplateUtils.getDefaultStylesCss()));
        list.add(StorageUtils.toStorageContent("styles.xml", DefaultTemplateUtils.getDefaultStylesXML(bdfServer)));
        if (withExtractionDef) {
            ExtractionDef extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, templateKey.getTransformationKey());
            if (extractionDef != null) {
                String extractionContent = ExtractionDefXMLPart.toString(extractionDef, true, false);
                list.add(StorageUtils.toStorageContent("extraction.xml", extractionContent));
            }
        }
        List<StorageContent> finalList = StorageUtils.wrap(list.toArray(new StorageContent[list.size()]));
        return new DefaultTemplateStorageUnit("xslt", templateKey, templateDef, finalList);
    }

    private static TemplateDef getTemplateDef(TemplateKey templateKey) {
        return TemplateDefBuilder.init(templateKey).toTemplateDef();
    }


    private static String getXslt(BdfServer bdfServer, TransformationKey transformationKey) {
        StringBuilder buf = new StringBuilder();
        OdtFicheXsltWriter ficheXsltWriter = new OdtFicheXsltWriter();
        ficheXsltWriter.setAppendable(buf);
        ficheXsltWriter.setIndentLength(0);
        try {
            ficheXsltWriter.appendXMLDeclaration();
            ficheXsltWriter.startXslStyleSheetOpenTag();
            OdXML.addDocumentNameSpaceAttributes(ficheXsltWriter);
            ficheXsltWriter.endOpenTag();
            if (transformationKey.isCorpusTransformationKey()) {
                ficheXsltWriter.addXslInclude("bdf://this/xslt/fiche_odt.xsl");
                SubsetKey corpusKey = transformationKey.toCorpusKey();
                Corpus corpus = (Corpus) bdfServer.getFichotheque().getSubset(corpusKey);
                ficheXsltWriter.add(bdfServer, corpus, false);
            } else if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
                ficheXsltWriter.addXslInclude("bdf://this/xslt/compilation_odt.xsl");
                for (Corpus corpus : bdfServer.getFichotheque().getCorpusList()) {
                    ficheXsltWriter.add(bdfServer, corpus, true);
                }
            }
            ficheXsltWriter.closeXslStyleSheet();
        } catch (IOException ioe) {
        }
        return buf.toString();
    }


}
