/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform;

import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.FormElement;
import fr.exemole.bdfserver.api.ficheform.FormElementProvider;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.tools.ficheform.builders.AddendaIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.AlbumIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.CorpusIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.GeopointProprieteSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.ImageProprieteSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.LangFieldElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.MontantInformationSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.MontantProprieteSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.PersonneProprieteSubfieldElementBuilder;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.utils.CorpusMetadataUtils;


/**
 *
 * @author Vincent Calame
 */
public final class FormElementProviderFactory {

    private FormElementProviderFactory() {
    }

    public static FormElementProvider newInstance(FicheFormParameters ficheFormParameters) {
        return new InternalFormElementProvider(ficheFormParameters);
    }


    private static class InternalFormElementProvider implements FormElementProvider {

        private final FicheFormParameters ficheFormParameters;

        public InternalFormElementProvider(FicheFormParameters ficheFormParameters) {
            this.ficheFormParameters = ficheFormParameters;
        }

        @Override
        public FormElement.Include getFormElement(FichePointeur fichePointeur, IncludeUi includeUi) {
            ExtendedIncludeKey extentedIncludeKey = includeUi.getExtendedIncludeKey();
            if (extentedIncludeKey == null) {
                switch (includeUi.getName()) {
                    case FichothequeConstants.LIAGE_NAME:
                        CorpusIncludeElementBuilder cieb = CorpusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                        if (cieb != null) {
                            return cieb.toCorpusIncludeElement();
                        } else {
                            return null;
                        }
                    case FichothequeConstants.RATTACHEMENT_NAME:
                        return null;
                    default:
                        return null;
                }
            } else {
                switch (extentedIncludeKey.getCategory()) {
                    case SubsetKey.CATEGORY_ADDENDA:
                        AddendaIncludeElementBuilder adieb = AddendaIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                        if (adieb != null) {
                            return adieb.toAddendaIncludeElement();
                        } else {
                            return null;
                        }
                    case SubsetKey.CATEGORY_ALBUM:
                        AlbumIncludeElementBuilder alieb = AlbumIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                        if (alieb != null) {
                            return alieb.toAlbumIncludeElement();
                        } else {
                            return null;
                        }
                    case SubsetKey.CATEGORY_CORPUS:
                        CorpusIncludeElementBuilder cieb = CorpusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                        if (cieb != null) {
                            return cieb.toCorpusIncludeElement();
                        } else {
                            return null;
                        }
                    case SubsetKey.CATEGORY_THESAURUS:
                        return FicheFormUtils.toThesaurusIncludeElement(fichePointeur, ficheFormParameters, includeUi);
                    default:
                        return null;
                }
            }
        }

        @Override
        public FormElement.Field getFormElement(FichePointeur fichePointeur, FieldUi fieldUi) {
            CorpusField corpusField = FicheFormUtils.getCorpusField(fichePointeur, fieldUi);
            if (corpusField == null) {
                return null;
            }
            if (corpusField.isGenerated()) {
                return null;
            }
            FieldKey fieldKey = corpusField.getFieldKey();
            if (fieldKey == FieldKey.ID) {
                return null;
            }
            if (fieldKey == FieldKey.LANG) {
                return LangFieldElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi)
                        .toLangFieldElement();
            }
            if (CorpusMetadataUtils.hasPersonneSubfields(corpusField)) {
                return PersonneProprieteSubfieldElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi)
                        .toPersonneProprieteSubfieldsElement();
            }
            if (CorpusMetadataUtils.hasGeopointSubfields(corpusField)) {
                return GeopointProprieteSubfieldsElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi)
                        .toGeopointProprieteSubfieldsElement();
            }
            if (CorpusMetadataUtils.isCurrenciesPropriete(corpusField)) {
                return MontantProprieteSubfieldsElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi)
                        .toMontantProprieteSubfieldsElement();
            }
            if (CorpusMetadataUtils.isCurrenciesInformation(corpusField)) {
                return MontantInformationSubfieldsElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi)
                        .toMontantInformationSubfieldsElement();
            }
            if (CorpusMetadataUtils.hasImageSubfields(corpusField)) {
                return ImageProprieteSubfieldsElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi)
                        .toImageProprieteSubfieldsElement();
            }
            return FicheFormUtils.toFormElement(fichePointeur, ficheFormParameters, corpusField, fieldUi);
        }

    }

}
