/* BdfServer_Commands - Copyright (c) 2011-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.album;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.imageio.IIOException;
import javax.imageio.ImageIO;
import net.fichotheque.album.Album;
import net.fichotheque.utils.AlbumUtils;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationFileUploadCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "IllustrationFileUpload";
    public final static String COMMANDKEY = "_ ALB-07";
    public final static String FILE_PARAMNAME = "file";
    private Album album;

    public IllustrationFileUploadCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return false;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        FileValue fileValue = requestMap.getFileValue(FILE_PARAMNAME);
        if (fileValue == null) {
            throw new BdfInstructionException(BdfInstructionUtils.emptyMandatoryParameter(FILE_PARAMNAME));
        }
        File tmpFile = null;
        try {
            tmpFile = saveFile(fileValue);
        } catch (IOException ioe) {
            setError("_ error.exception.io", ioe.getMessage());
        } catch (ErrorMessageException eme) {
            setCommandMessage(eme.getErrorMessage());
        }
        if (tmpFile != null) {
            setDone("_ done.album.illustrationfileupload");
            putResultObject(TMPFILE_OBJ, tmpFile);
            putResultObject(ALBUM_OBJ, album);
        }
        fileValue.free();
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        album = BdfInstructionUtils.getMandatoryAlbum(fichotheque, requestMap);
        return true;
    }

    private File saveFile(FileValue fileValue) throws IOException, ErrorMessageException {
        long length = fileValue.length();
        if (length < 2) {
            setError("_ error.empty.file");
            return null;
        }
        String extension = AlbumUtils.checkExtension(fileValue.getName());
        if (extension == null) {
            setError("_ error.wrong.imageextension");
            return null;
        }
        try (InputStream is = fileValue.getInputStream()) {
            return saveFile(bdfServer, is, extension, album);
        }
    }

    public static File saveFile(BdfServer bdfServer, InputStream inputStream, String extension, Album album) throws IOException, ErrorMessageException {
        File tmpFile;
        if (AlbumUtils.convertToPng(extension)) {
            extension = "png";
            tmpFile = AlbumCommandUtils.getTmpFile(bdfServer, "png");
            BufferedImage srcImg = ImageIO.read(inputStream);
            if (srcImg == null) {
                throw new ErrorMessageException("_ error.unsupported.imagetype");
            } else {
                ImageIO.write(srcImg, "png", tmpFile);
            }
        } else {
            byte[] array = IOUtils.toByteArray(inputStream);
            try {
                BufferedImage srcImg = ImageIO.read(new ByteArrayInputStream(array));
                if (srcImg == null) {
                    throw new ErrorMessageException("_ error.unsupported.imagetype");
                }
            } catch (IOException ioe) {
                if (ioe instanceof IIOException) {
                    throw new ErrorMessageException("_ error.unsupported.imagetype");
                } else {
                    throw new ErrorMessageException("_ error.exception.io", ioe.getMessage());
                }
            }
            tmpFile = AlbumCommandUtils.getTmpFile(bdfServer, extension);
            try (OutputStream os = new FileOutputStream(tmpFile)) {
                IOUtils.copy(new ByteArrayInputStream(array), os);
            }
        }
        AlbumCommandUtils.minimizeTmpFile(tmpFile, album);
        return tmpFile;
    }

}
