/* BdfServer_Email - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email.datasource;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.util.Map;
import javax.activation.DataSource;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.exportation.transformation.SimpleTemplate;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.ExtractionSource;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.tools.extraction.builders.BuilderUtils;
import net.fichotheque.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.io.IOUtils;


/**
 *
 * @author Vincent Calame
 */
public class SimpleTemplateDataSource implements DataSource {

    private final String name;
    private final String mimeType;
    private final String charset;
    private final String content;

    private SimpleTemplateDataSource(String name, String mimeType, String charset, String content) {
        this.name = name;
        this.mimeType = mimeType;
        this.charset = charset;
        this.content = content;
    }


    @Override
    public String getContentType() {
        return mimeType + ";charset=" + charset;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public InputStream getInputStream() throws IOException {
        return IOUtils.toInputStream(content, charset);
    }

    @Override
    public OutputStream getOutputStream() throws IOException {
        throw new IOException("no editable source");
    }

    public String getMimeType() {
        return mimeType;
    }

    public String getContent() {
        return content;
    }

    public static SimpleTemplateDataSource newFicheInstance(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, FicheMeta ficheMeta, @Nullable String templateName) {
        SubsetKey corpusKey = ficheMeta.getCorpus().getSubsetKey();
        String baseName = corpusKey.getSubsetName() + "-" + String.valueOf(ficheMeta.getId());
        TemplateKey templateKey = getTemplateKey(new TransformationKey(corpusKey), templateName, bdfUser);
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        return transform(bdfServer, bdfUser, templateKey, baseName, extractionContext, ficheMeta);
    }

    public static SimpleTemplateDataSource newCompilationInstance(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, Fiches fiches, String templateName) {
        String baseName = "compilation";
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .setSelectedFiches(fiches)
                .toExtractionContext();
        TemplateKey templateKey = getTemplateKey(TransformationKey.COMPILATION_TRANSFORMATIONKEY, templateName, bdfUser);
        return transform(bdfServer, bdfUser, templateKey, baseName, extractionContext, null);
    }

    private static TemplateKey getTemplateKey(TransformationKey transformationKey, String templateName, BdfUser bdfUser) {
        if (templateName == null) {
            templateName = BdfUserUtils.getSimpleTemplateName(bdfUser, transformationKey);
        }
        TemplateKey templateKey;
        try {
            templateKey = TemplateKey.parse(transformationKey, templateName);
        } catch (ParseException pe) {
            templateKey = TemplateKey.toDefault(transformationKey);
        }
        return templateKey;
    }

    private static SimpleTemplateDataSource transform(BdfServer bdfServer, BdfUser bdfUser, TemplateKey templateKey, String baseName, ExtractionContext extractionContext, Object dynamicObject) {
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
        if (simpleTemplate == null) {
            simpleTemplate = transformationManager.getSimpleTemplate(templateKey.toDefaultTemplateKey());
        }
        ExtractionDef extractionDef = simpleTemplate.getCustomExtractionDef();
        if (extractionDef == null) {
            extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, templateKey.getTransformationKey());
        }
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        String extractionString = ExtractionXMLPart.toString(extractionContext, extractionDef, extractionSource, false);
        Map<String, Object> paramMap = BdfTransformationUtils.newParamMap(bdfServer, bdfUser);
        paramMap.put(TransformationConstants.INCLUDESCRIPTS_PARAMETER, "");
        String mimeType = simpleTemplate.getMimeType();
        String extension = bdfServer.getMimeTypeResolver().getPreferredExtension(mimeType);
        if (extension == null) {
            extension = "html";
        }
        String content = simpleTemplate.transform(extractionString, paramMap);
        return new SimpleTemplateDataSource(baseName + "." + extension, mimeType, simpleTemplate.getCharset(), content);
    }

}
