/* FichothequeLib_API - Copyright (c) 2020-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils.selection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetItem;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.selection.RedacteurQuery;
import net.fichotheque.selection.RedacteurSelector;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.utils.EligibilityUtils;
import net.mapeadores.util.localisation.Lang;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurSelectorBuilder {

    private final Fichotheque fichotheque;
    private final Lang lang;
    private final List<RedacteurSelectEngine> list = new ArrayList<RedacteurSelectEngine>();
    private Predicate<SubsetItem> redacteurPredicate = EligibilityUtils.ALL_SUBSETITEM_PREDICATE;


    public RedacteurSelectorBuilder(Fichotheque fichotheque, Lang lang) {
        this.fichotheque = fichotheque;
        this.lang = lang;
    }

    public RedacteurSelectorBuilder setRedacteurPredicate(Predicate<SubsetItem> redacteurPredicate) {
        if (redacteurPredicate == null) {
            this.redacteurPredicate = EligibilityUtils.ALL_SUBSETITEM_PREDICATE;
        } else {
            this.redacteurPredicate = redacteurPredicate;
        }
        return this;
    }

    public RedacteurSelectorBuilder add(RedacteurQuery redacteurQuery) {
        list.add(new RedacteurSelectEngine(fichotheque, redacteurQuery.getNomcompletCondition(), lang));
        return this;
    }

    public RedacteurSelectorBuilder addAll(Collection<RedacteurQuery> collection) {
        for (RedacteurQuery query : collection) {
            add(query);
        }
        return this;
    }

    public RedacteurSelector toRedacteurSelector() {
        RedacteurSelectEngine[] array = list.toArray(new RedacteurSelectEngine[list.size()]);
        return new InternalRedacteurSelector(redacteurPredicate, array);
    }

    public static RedacteurSelectorBuilder init(Fichotheque fichotheque, Lang lang) {
        return new RedacteurSelectorBuilder(fichotheque, lang);
    }


    private static class InternalRedacteurSelector implements RedacteurSelector {

        private final Predicate<SubsetItem> redacteurPredicate;
        private final RedacteurSelectEngine[] array;

        private InternalRedacteurSelector(Predicate<SubsetItem> redacteurPredicate, RedacteurSelectEngine[] array) {
            this.redacteurPredicate = redacteurPredicate;
            this.array = array;
        }

        @Override
        public boolean test(Redacteur redacteur) {
            if (!(redacteurPredicate.test(redacteur))) {
                return false;
            }
            for (RedacteurSelectEngine engine : array) {
                if (engine.isSelected(redacteur)) {
                    return true;
                }
            }
            return false;
        }

        @Override
        public Croisement isSelected(Redacteur redacteur, Croisement croisement) {
            if (test(redacteur)) {
                return croisement;
            } else {
                return null;
            }
        }

    }

}
